<div class="audio-player audio-player-initialising">
    <div class="player-button">
        <i class="icon audio-play-icon"></i>
    </div>
    <div class="player-time player-current-time">00:00</div>
    <div class="player-seekbar">
        <div class="progress">
            <div class="progress-pin"></div>
            <div class="progress-bar" role="progressbar" aria-valuemin="0" aria-valuemax="100" style="width: 0;"></div>
        </div>
    </div>
    <div class="player-time player-end-time">--:--</div>
</div>